package digital.laboratory.platform.reagent.controller;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import digital.laboratory.platform.common.core.util.R;
import digital.laboratory.platform.common.log.annotation.SysLog;
import digital.laboratory.platform.common.mybatis.security.service.DLPUser;
import digital.laboratory.platform.common.oss.service.OssFile;
import digital.laboratory.platform.reagent.dto.ReagentConsumableInventoryDTO;
import digital.laboratory.platform.reagent.entity.ReagentConsumableInventory;
import digital.laboratory.platform.reagent.entity.ReferenceMaterial;
import digital.laboratory.platform.reagent.service.ReagentConsumableInventoryService;
import digital.laboratory.platform.reagent.service.ReferenceMaterialService;
import digital.laboratory.platform.reagent.utils.QRCodeUtils;
import digital.laboratory.platform.reagent.vo.*;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.security.oauth2.provider.OAuth2Authentication;
import org.springframework.web.bind.annotation.*;

import javax.imageio.ImageIO;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.awt.image.BufferedImage;
import java.io.IOException;
import java.security.Principal;
import java.util.List;

/**
 * 试剂耗材库存
 *
 * @author Zhang Xiaolong created at  2023-03-10
 * @describe 试剂耗材库存 前端控制器
 * <p>
 * 这是与表示层的接口, 不应该接业务逻辑写在这里, 业务逻辑应该写在 service 中
 * 这里写什么:
 * 为前端提供数据, 接受前端的数据
 * 为前端提供的数据, 从 service 取得后, 可以做一些适当的加工, 这种加工不是业务层面的, 只能是数据格式上, 为方便前端处理
 * 接受前端的数据, 每一个函数的参数可以先做一些整理后, 再调用 service 中的函数。这里对参数的整理, 应该只是格式上的, 而不能是业务上的
 * 数据层在 mapper 中, 数据层不涉及业务, 只管技术上的 对象<->表 之间的转换
 */
@RestController
@RequiredArgsConstructor
@RequestMapping("/reagent_consumable_inventory")
@Api(value = "reagent_consumable_inventory", tags = "试剂耗材库存管理")
public class ReagentConsumableInventoryController {

    private final ReagentConsumableInventoryService reagentConsumableInventoryService;

    private final ReferenceMaterialService referenceMaterialService;

    private final OssFile ossFile;

    /**
     * 通过id查询试剂耗材库存
     *
     * @param reagentConsumableInventoryId id
     * @return R
     */
    @ApiOperation(value = "通过id查询", notes = "通过id查询")
    @GetMapping("/{reagentConsumableInventoryId}")
//    @PreAuthorize("@pms.hasPermission('reagent_reagent_consumable_inventory_get')")
    public R<ReagentConsumableInventory> getById(@PathVariable("reagentConsumableInventoryId") String reagentConsumableInventoryId, HttpServletRequest theHttpServletRequest) {
        Principal principal = theHttpServletRequest.getUserPrincipal();
        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        ReagentConsumableInventory reagentConsumableInventory = reagentConsumableInventoryService.getById(reagentConsumableInventoryId);
        return R.ok(reagentConsumableInventory);
        //return R.ok(reagentConsumableInventoryService.getById(reagentConsumableInventoryId));
    }


    /**
     * @param
     * @return
     */
    @ApiOperation(value = "标准物质列表", notes = "标准物质列表")
    @GetMapping("/RList")
//    @PreAuthorize("@pms.hasPermission('reagent_reagent_consumable_inventory_get')")
    public R<Page<ReagentConsumableInventoryFullVO>> getReagentConsumableInventoryPage(Integer current, Integer size, String reagentConsumableName, Integer referenceMaterialStatus, HttpServletRequest theHttpServletRequest) {
        Principal principal = theHttpServletRequest.getUserPrincipal();
        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        QueryWrapper<ReagentConsumableInventory> reagentConsumableInventoryQueryWrapper = new QueryWrapper<>();
        QueryWrapper<ReferenceMaterial> referenceMaterialQueryWrapper = new QueryWrapper<>();

        String number = reagentConsumableName;

        if (reagentConsumableName != null) {

            reagentConsumableInventoryQueryWrapper.like("reagent_consumable_name", reagentConsumableName).or()
                    .like("species", reagentConsumableName);
        }


        Page<ReagentConsumableInventoryFullVO> allList = reagentConsumableInventoryService.getAllList(current, size, reagentConsumableInventoryQueryWrapper, referenceMaterialStatus, number);
        return R.ok(allList);
//		return R.ok(reagentConsumableInventoryService.page(page, Wrappers.query(reagentConsumableInventory)));
    }


    @ApiOperation(value = "标准物质管理列表", notes = "标准物质管理列表")
    @GetMapping("/standardList")
//    @PreAuthorize("@pms.hasPermission('reagent_reagent_consumable_inventory_get')")
    public R<IPage<ReagentConsumableInventoryVO>> getReagentConsumableInventoryList(Page<ReagentConsumableInventory> page, Integer warning, String remark, String reagentConsumableName, HttpServletRequest theHttpServletRequest) {
        Principal principal = theHttpServletRequest.getUserPrincipal();
        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();
        IPage<ReagentConsumableInventoryVO> reagentConsumableInventoryRMVOList = reagentConsumableInventoryService.getReagentConsumableInventoryRMVOList(page, Wrappers.<ReagentConsumableInventory>query()

                .like(StrUtil.isNotBlank(reagentConsumableName), "reagent_consumable_name", reagentConsumableName)
                .like(warning != null && warning == 1, "warning_information", "库存不足")
                .eq("category", "标准物质").or()

                .like(StrUtil.isNotBlank(reagentConsumableName), "remark", reagentConsumableName)
                .like(warning != null && warning == 1, "warning_information", "库存不足")
                .eq("category", "标准物质").or()

                .like(StrUtil.isNotBlank(reagentConsumableName), "reagent_consumable_name", reagentConsumableName)
                .like(warning != null && warning == 1, "warning_information", "库存不足")
                .eq("category", "标准储备溶液").or()

                .like(StrUtil.isNotBlank(reagentConsumableName), "remark", reagentConsumableName)
                .like(warning != null && warning == 1, "warning_information", "库存不足")
                .eq("category", "标准储备溶液").or()

                .like(StrUtil.isNotBlank(reagentConsumableName), "species", reagentConsumableName)
                .like(warning != null && warning == 1, "warning_information", "库存不足")
                .eq("category", "标准物质").or().

                like(StrUtil.isNotBlank(reagentConsumableName), "species", reagentConsumableName)
                .like(warning != null && warning == 1, "warning_information", "库存不足")
                .eq("category", "标准储备溶液").or(), warning);


        return R.ok(reagentConsumableInventoryRMVOList);
//		return R.ok(reagentConsumableInventoryService.page(page, Wrappers.query(reagentConsumableInventory)));
    }

    @ApiOperation(value = "试剂耗材管理列表", notes = "试剂耗材管理列表")
    @GetMapping("/List")
//    @PreAuthorize("@pms.hasPermission('reagent_reagent_consumable_inventory_get')")
    public R<IPage<ReagentConsumableInventoryVO>> getList(Page page, String reagentConsumableName, Integer warning, String remark, HttpServletRequest theHttpServletRequest) {
        Principal principal = theHttpServletRequest.getUserPrincipal();
        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();
        IPage<ReagentConsumableInventoryVO> reagentConsumableInventoryRMVOList = reagentConsumableInventoryService.getReagentConsumableInventoryREVOList(page, Wrappers.<ReagentConsumableInventory>query()

                .like(StrUtil.isNotBlank(reagentConsumableName), "reagent_consumable_name", reagentConsumableName)
                .like(warning != null && warning == 1, "warning_information", "库存不足")
                .eq("category", "试剂").or()

                .like(StrUtil.isNotBlank(reagentConsumableName), "remark", reagentConsumableName)
                .like(warning != null && warning == 1, "warning_information", "库存不足")
                .eq("category", "试剂").or()

                .like(StrUtil.isNotBlank(reagentConsumableName), "reagent_consumable_name", reagentConsumableName)
                .like(warning != null && warning == 1, "warning_information", "库存不足")
                .eq("category", "耗材").or()

                .like(StrUtil.isNotBlank(reagentConsumableName), "remark", reagentConsumableName)
                .like(warning != null && warning == 1, "warning_information", "库存不足")
                .eq("category", "耗材")

                .like(StrUtil.isNotBlank(reagentConsumableName), "reagent_consumable_name", reagentConsumableName)
                .like(warning != null && warning == 1, "species", "库存不足")
                .eq("category", "试剂").or()

                .like(StrUtil.isNotBlank(reagentConsumableName), "remark", reagentConsumableName)
                .like(warning != null && warning == 1, "species", "库存不足")
                .eq("category", "耗材").or(), warning);
        return R.ok(reagentConsumableInventoryRMVOList);
//		return R.ok(reagentConsumableInventoryService.page(page, Wrappers.query(reagentConsumableInventory)));
    }

    /**
     * 分页查询
     *
     * @return
     */
    @ApiOperation(value = "试剂耗材列表", notes = "试剂耗材列表")
    @GetMapping("/MList")
//    @PreAuthorize("@pms.hasPermission('reagent_reagent_consumable_inventory_get')")
    public R<IPage<ReagentConsumableInventoryFullVO>> getReferenceMaterialVOList(Page page, String reagentConsumableName, HttpServletRequest theHttpServletRequest) {
        Principal principal = theHttpServletRequest.getUserPrincipal();
        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        QueryWrapper<ReagentConsumableInventory> reagentConsumableInventoryQueryWrapper = new QueryWrapper<>();

        IPage<ReagentConsumableInventoryFullVO> allRM = reagentConsumableInventoryService.getAllRM(page, reagentConsumableInventoryQueryWrapper
                .eq("category", "试剂")
                .like(!StrUtil.isEmpty(reagentConsumableName), "reagent_consumable_name", reagentConsumableName).or()
                .eq("category", "试剂")
                .like(!StrUtil.isEmpty(reagentConsumableName), "species", reagentConsumableName).or()
                .eq("category", "耗材")
                .like(!StrUtil.isEmpty(reagentConsumableName), "reagent_consumable_name", reagentConsumableName).or()
                .eq("category", "耗材")
                .like(!StrUtil.isEmpty(reagentConsumableName), "species", reagentConsumableName));


        return R.ok(allRM);
    }

    /**
     * 分页查询
     *
     * @return
     */
    @ApiOperation(value = "试剂耗材/标准物质集合列表", notes = "试剂耗材/标准物质集合列表")
    @GetMapping("/full")
//    @PreAuthorize("@pms.hasPermission(' ')")
    public R<List<ReagentConsumableInventoryFullVO>> getReagentConsumableInventoryFull(String category, String name, HttpServletRequest theHttpServletRequest) {
        Principal principal = theHttpServletRequest.getUserPrincipal();
        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        List<ReagentConsumableInventoryFullVO> reagentConsumableInventoryFull = reagentConsumableInventoryService.getReagentConsumableInventoryFull(Wrappers.<ReagentConsumableInventory>query()
                .eq((StrUtil.isNotBlank(category)), "category", category)
                .like((StrUtil.isNotBlank(name)), "reagent_consumable_name", name));

        return R.ok(reagentConsumableInventoryFull);
    }


    /**-------标准物质一览表打印*/
    /**
     * 标准物质一览表打印
     *
     * @param
     * @return
     */
    @ApiOperation(value = "标准物质一览表打印", notes = "标准物质一览表打印")
    @SysLog("标准物质一览表打印")
    @PostMapping("/print")
    @PreAuthorize("@pms.hasPermission('reagent_reagent_consumable_inventory_print')")
    public void bizGetPDFInventory(HttpServletRequest theHttpServletRequest, HttpServletResponse httpServletResponse) {
        System.out.println("bizApplyWord.................");
        Principal principal = theHttpServletRequest.getUserPrincipal();
        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        String id = IdWorker.get32UUID().toUpperCase();

        List<ReferenceMaterialVO> voList = referenceMaterialService.getReferenceMaterialList();

        if (voList.isEmpty()) {
            throw new RuntimeException("未查询到相关信息");
        }


        String applyFileName = "标准物质一览表-" + id;

        String pdfFilePath = "document" + "/" + "referenceMaterial" + "/" + id + "/" + applyFileName + ".pdf";
        try {
            //直接调用pdf方法
            reagentConsumableInventoryService.referenceMaterialTablePDF(voList, id, theHttpServletRequest, httpServletResponse);
            ossFile.fileGet(pdfFilePath, httpServletResponse.getOutputStream());
            httpServletResponse.setContentType(applyFileName);
        } catch (Exception e) {
            System.out.println(String.format("minioFile objectExist() Exception. %s", e.getLocalizedMessage()));
            e.printStackTrace();
        }
    }

    /**
     * 标准物质打印二维码
     *
     * @param code QRCODE字符串
     * @return R<CaseEventVO>
     */
    @ApiOperation(value = "标准物质打印二维码", notes = "标准物质打印二维码")
    @GetMapping("/code")
//    @PreAuthorize("@pms.hasAnyPermission('reagent_reagent_consumable_inventory_get')")

    public String getBarCodeImageBase64(String code, HttpServletResponse httpServletResponse) throws IOException {

        return reagentConsumableInventoryService.buildCodeLabelContent(code);


    }

    /**
     * 标准储备溶液打印二维码
     *
     * @param code QRCODE字符串
     */
    @ApiOperation(value = "标准储备溶液打印二维码", notes = "标准储备溶液打印二维码")
    @GetMapping("/codeSolution")
//    @PreAuthorize("@pms.hasAnyPermission('reagent_reagent_consumable_inventory_get')")

    public String printSolutionTag(String code, HttpServletResponse httpServletResponse) throws IOException {

        return reagentConsumableInventoryService.printSolutionTag(code);


    }

    /**
     * 扫码录入物品编码
     *
     * @param code 物品二维码
     */
    @ApiOperation(value = "扫码录入物品编码", notes = "扫码录入物品编码")
    @PutMapping("/code")
    @PreAuthorize("@pms.hasPermission('reagent_reagent_consumable_inventory_get')")
    public R<String> setCode(String id, String code, HttpServletResponse httpServletResponse) {

        reagentConsumableInventoryService.setCode(id, code);

        return R.ok("录入成功");
    }

    /**
     * 录入标准物质自带的二维码信息
     *
     * @param code 物品二维码
     * @param id   标准物质ID
     */
    @ApiOperation(value = "录入标准物质自带的二维码信息", notes = "录入标准物质自带的二维码信息")
    @PutMapping("/RMCode")
    @PreAuthorize("@pms.hasPermission('reagent_reagent_consumable_inventory_get')")
    public R<String> setRMCode(String id, String code, HttpServletResponse httpServletResponse) {

        reagentConsumableInventoryService.setRMCode(id, code);

        return R.ok("录入成功");
    }

    /**
     * 通过扫码，获取标准物质所有信息
     *
     * @param id id
     */
    @ApiOperation(value = "", notes = "通过id，获取标准物质所有信息")
    @GetMapping("/getByCode")
    @PreAuthorize("@pms.hasPermission('reagent_reagent_consumable_inventory_get_by_code')")
    public R<ReagentConsumableInventoryFullVO> getByCode(String id, String number, HttpServletResponse httpServletResponse) {

        if (StrUtil.isNotBlank(number)) {

            ReferenceMaterial referenceMaterial = referenceMaterialService.getOne(Wrappers.<ReferenceMaterial>query().eq("number", number));

            if (referenceMaterial != null) {

                return R.ok(reagentConsumableInventoryService.getByCode(referenceMaterial.getId()));
            } else {

                return R.failed("未能查询到该标准物质的详细信息");
            }
        }
        ReagentConsumableInventoryFullVO byCode = reagentConsumableInventoryService.getByCode(id);
        return R.ok(byCode);
    }


    /**
     * 获取用户列表
     */
    @ApiOperation(value = "", notes = "获取用户列表")
    @GetMapping("/userList")
    @PreAuthorize("@pms.hasPermission('reagent_standard_material_approval_form_add')")
    public R<List<UserVO>> getUserList(HttpServletResponse httpServletResponse) {

        return R.ok(reagentConsumableInventoryService.getUserList());
    }

    @PostMapping("inventory/entry")
    public R<String> inventoryEntry(@RequestBody List<ReagentConsumableInventoryDTO> reagentConsumableInventoryDTOList, HttpServletResponse httpServletResponse) {

        reagentConsumableInventoryService.inventoryEntry(reagentConsumableInventoryDTOList);

        return R.ok("导入成功");

    }


}
