package digital.laboratory.platform.reagent.service.impl;

import cn.hutool.core.date.DateTime;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import digital.laboratory.platform.common.mybatis.security.service.DLPUser;
import digital.laboratory.platform.reagent.dto.AuditAndApproveDTO;
import digital.laboratory.platform.reagent.dto.PurchasingPlanDTO;
import digital.laboratory.platform.reagent.entity.*;
import digital.laboratory.platform.reagent.mapper.PurchasingPlanMapper;
import digital.laboratory.platform.reagent.service.*;
import digital.laboratory.platform.reagent.vo.ProcurementContentVO;
import digital.laboratory.platform.reagent.vo.PurchasingPlanVO;
import digital.laboratory.platform.reagent.vo.ReplacementVO;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;

/**
 * (采购计划)服务实现类
 *
 * @author Zhang Xiaolong created at  2023-03-10
 * @describe (采购计划) 服务实现类
 */
@Service
@SuppressWarnings("all")
public class PurchasingPlanServiceImpl extends ServiceImpl<PurchasingPlanMapper, PurchasingPlan> implements PurchasingPlanService {
    @Autowired
    private ProcurementContentService procurementContentService;

    @Autowired
    private DetailsOfCentralizedService detailsOfCentralizedService;

    @Autowired
    private ReagentConsumablesService reagentConsumablesService;

    @Autowired
    private PurchaseListService purchaseListService;

    @Autowired
    private CentralizedRequestService requestService;

    @Autowired
    private PurchaseListDetailsService purchaseListDetailsService;

    @Autowired
    private WarehousingContentService warehousingContentService;

    @Autowired
    private SupplierInformationService supplierInformationService;

    @Override//分页查询
    public IPage<PurchasingPlanVO> getPurchasingPlanVOPage(IPage<PurchasingPlan> page, QueryWrapper<PurchasingPlan> qw) {
        IPage<PurchasingPlanVO> purchasingPlanVOPage = baseMapper.getPurchasingPlanVOPage(page, qw);
        return purchasingPlanVOPage;
    }

    @Override//采购计划集合
    public List<PurchasingPlanVO> getPurchasingPlanVOList(QueryWrapper<PurchasingPlan> qw) {
        return baseMapper.getPurchasingPlanVOList(qw);
    }

    @Override//通过ID查询
    public PurchasingPlanVO getPurchasingPlanVO(String purchasingPlanId) {

        PurchasingPlanVO purchasingPlanVO = baseMapper.getPurchasingPlanVO(purchasingPlanId);
        if (purchasingPlanVO == null) {
            throw new RuntimeException(String.format("信息不存在"));
        }
        List<ProcurementContentVO> procurementContentVOList = procurementContentService.getProcurementContentVOList(purchasingPlanId);//将采购计划明细Set至采购计划VO
        purchasingPlanVO.setProcurementContentVOList(procurementContentVOList);
        return purchasingPlanVO;
    }

    @Transactional
    @Override//创建计划
    public PurchasingPlanVO addById(List<PurchasingPlanDTO> purchasingPlanDTOList, DLPUser user) {

        PurchasingPlan purchasingPlan = new PurchasingPlan();//创建采购计划
        purchasingPlan.setPurchasingPlanId(IdWorker.get32UUID().toUpperCase());
        purchasingPlan.setCreateId(user.getId());//创建人ID
        purchasingPlan.setStatus(0);//状态
        purchasingPlan.setCreateName(user.getName());//创建人名称
        List<ProcurementContent> list = new ArrayList<>();//需要整合的集合
        boolean flag = true;

        for (PurchasingPlanDTO purchasingPlanDTO : purchasingPlanDTOList) {
            //将列入计划的集中采购申请状态变为3
            CentralizedRequest centralizedRequest = requestService.getById(detailsOfCentralizedService.getById(purchasingPlanDTO.getDetailsOfCentralizedId()).getCentralizedRequestId());
            centralizedRequest.setStatus(3);
            centralizedRequest.setPurchasingPlanId(purchasingPlan.getPurchasingPlanId());
            requestService.updateById(centralizedRequest);

            DetailsOfCentralized detailsOfCentralized = detailsOfCentralizedService.getById(purchasingPlanDTO.getDetailsOfCentralizedId());//集中采购申请明细
            ReagentConsumables reagentConsumables = reagentConsumablesService.getById(detailsOfCentralized.getReagentConsumableId());//试剂耗材信息

            flag = true;
            //合并相同的采购内容，若存在相同的采购物品，则无需创建计划明细，将采购申请明细与该计划明细关联即可
            if (list.size() != 0) {

                for (int i = 0; i < list.size(); i++) {

                    if (list.get(i).getReagentConsumableId().equals(purchasingPlanDTO.getReagentConsumableId())) {

                        list.get(i).setUnitPrice(reagentConsumables.getUnitPrice());//获取单价
                        //Set采购数量
                        list.get(i).setQuantityPurchased(list.get(i).getQuantityPurchased() + purchasingPlanDTO.getNumberOfApplications());//合并之前的数量加上被合并对象的数量
                        list.get(i).setSubtotal(list.get(i).getQuantityPurchased() * list.get(i).getUnitPrice());//计算小计

                        //添加采购申请明细的采购数量

                        detailsOfCentralized.setQuantityPurchased(detailsOfCentralized.getNumberOfApplications());
                        detailsOfCentralized.setProcurementContentId(list.get(i).getProcurementContentId());
                        detailsOfCentralizedService.updateById(detailsOfCentralized);
                        //被合并后，改变flag，进入下一个循环
                        flag = false;
                    }
                }
                //无需合并的采购计划明细
                if (flag) {
                    ProcurementContent procurementContent = new ProcurementContent();//新建计划明细
                    BeanUtils.copyProperties(purchasingPlanDTO, procurementContent);//拷贝
                    procurementContent.setProcurementContentId(IdWorker.get32UUID().toUpperCase());
                    procurementContent.setPurchasingPlanId(purchasingPlan.getPurchasingPlanId());//关联计划ID
                    procurementContent.setQuantityPurchased(purchasingPlanDTO.getNumberOfApplications());//赋值采购数量
                    detailsOfCentralized.setQuantityPurchased(detailsOfCentralized.getNumberOfApplications());//赋值采购数量
                    procurementContent.setUnitPrice(reagentConsumables.getUnitPrice());//单价
                    procurementContent.setSubtotal(procurementContent.getUnitPrice() * procurementContent.getQuantityPurchased());//小计
                    detailsOfCentralized.setProcurementContentId(procurementContent.getProcurementContentId());//采购申请明细关联采购计划明细

                    detailsOfCentralizedService.updateById(detailsOfCentralized);
                    list.add(procurementContent);//将计划明细添加至集合

                }

            } else {//对于首次循环，List一定为空，故执行下列语句，且只会执行一次
                ProcurementContent procurementContent = new ProcurementContent();
                BeanUtils.copyProperties(purchasingPlanDTO, procurementContent);
                DetailsOfCentralized byId = detailsOfCentralizedService.getById(purchasingPlanDTO.getDetailsOfCentralizedId());
                byId.setQuantityPurchased(byId.getNumberOfApplications());
                procurementContent.setProcurementContentId(IdWorker.get32UUID().toUpperCase());
                procurementContent.setPurchasingPlanId(purchasingPlan.getPurchasingPlanId());
                procurementContent.setQuantityPurchased(procurementContent.getNumberOfApplications());
                procurementContent.setUnitPrice(reagentConsumables.getUnitPrice());
                procurementContent.setSubtotal(procurementContent.getQuantityPurchased() * procurementContent.getUnitPrice());
                byId.setProcurementContentId(procurementContent.getProcurementContentId());
                detailsOfCentralizedService.updateById(byId);
                list.add(procurementContent);
            }
        }

        if (this.save(purchasingPlan) & procurementContentService.saveBatch(list)) {
            calculatedAmount(purchasingPlan.getPurchasingPlanId());//计算经费预算
            PurchasingPlanVO purchasingPlanVO = this.getPurchasingPlanVO(purchasingPlan.getPurchasingPlanId());//获取VO
            return purchasingPlanVO;
        } else throw new RuntimeException(String.format("保存失败"));
    }

//    @Override//新增明细
//    public ProcurementContent addContent(PurchasingPlanDTO purchasingPlanDTO) {
//
//        ProcurementContent procurementContent = new ProcurementContent();
//
//        BeanUtils.copyProperties(purchasingPlanDTO, procurementContent);
//
//        if (procurementContentService.save(procurementContent)) {
//            return procurementContent;
//        } else {
//            return null;
//        }
//    }

    @Override
    @Transactional//修改采购计划明细（已废弃）
    public ProcurementContent editById(PurchasingPlanDTO purchasingPlanDTO) {

        ProcurementContent byId = procurementContentService.getById(purchasingPlanDTO.getProcurementContentId());

        BeanUtils.copyProperties(purchasingPlanDTO, byId);

        byId.setSubtotal(byId.getUnitPrice() * byId.getQuantityPurchased());

        LambdaQueryWrapper<DetailsOfCentralized> detailsOfCentralizedLambdaQueryWrapper = new LambdaQueryWrapper<>();

        detailsOfCentralizedLambdaQueryWrapper.eq(DetailsOfCentralized::getId, byId.getDetailsOfCentralizedId());

        DetailsOfCentralized detailsOfCentralized = detailsOfCentralizedService.getOne(detailsOfCentralizedLambdaQueryWrapper);

        detailsOfCentralized.setQuantityPurchased(byId.getQuantityPurchased());

        procurementContentService.updateById(byId);

        detailsOfCentralizedService.updateById(detailsOfCentralized);

        calculatedAmount(byId.getPurchasingPlanId());

        return byId;

    }


    @Override//修改整合的采购计划明细中的采购申请明细

    public ProcurementContent editDetails(PurchasingPlanDTO purchasingPlanDTO) {

        ProcurementContent procurementContent = procurementContentService.getById(purchasingPlanDTO.getProcurementContentId());//得到计划明细

        procurementContent.setUnitPrice(purchasingPlanDTO.getUnitPrice());//得到修改的明细单价

        PurchasingPlan purchasingPlan = this.getById(procurementContent.getPurchasingPlanId());//查询出采购计划

        DetailsOfCentralized detailsOfCentralized = detailsOfCentralizedService.getById(purchasingPlanDTO.getDetailsOfCentralizedId());//获取修改的对应的采购申请明细

        if (detailsOfCentralized.getQuantityPurchased() == null) {
            //原本合并总数量-采购申请明细数量+采购申请明细修改后的数量
            procurementContent.setQuantityPurchased(procurementContent.getQuantityPurchased() - detailsOfCentralized.getNumberOfApplications() + purchasingPlanDTO.getQuantityPurchased());

        } else {//原本合并总数量-采购申请修改后的采购数量+修改数量
            procurementContent.setQuantityPurchased(procurementContent.getQuantityPurchased() - detailsOfCentralized.getQuantityPurchased() + purchasingPlanDTO.getQuantityPurchased());
        }
        procurementContent.setSubtotal(procurementContent.getUnitPrice() * procurementContent.getQuantityPurchased());//计算小计
        detailsOfCentralized.setQuantityPurchased(purchasingPlanDTO.getQuantityPurchased());//赋值采购实际数量
        detailsOfCentralized.setDetailsRemark(purchasingPlanDTO.getPlanRemark());//修改备注

        if (procurementContentService.updateById(procurementContent) && detailsOfCentralizedService.updateById(detailsOfCentralized)) {
            calculatedAmount(procurementContent.getPurchasingPlanId());//计算经费预算
            return procurementContent;
        } else throw new RuntimeException(String.format("修改失败"));
    }


    @Override
    @Transactional//删除一条计划明细
    public Boolean delContentById(String procurementContentId) {

        ProcurementContent procurementContent = procurementContentService.getById(procurementContentId);//采购计划明细
        //该采购计划明细关联的集中采购申请明细
        List<DetailsOfCentralized> list = detailsOfCentralizedService.list(Wrappers.<DetailsOfCentralized>query().eq("procurement_content_id", procurementContentId));
        for (DetailsOfCentralized detailsOfCentralized : list) {
            detailsOfCentralized.setQuantityPurchased(0);//采购申请明细的采购数量
        }
        PurchasingPlan purchasingPlan = this.getById(procurementContent.getPurchasingPlanId());//采购计划

        if (this.getById(procurementContent.getPurchasingPlanId()).getStatus() == 0
                && detailsOfCentralizedService.updateBatchById(list) && procurementContentService.removeById(procurementContentId)) {

            List<ProcurementContent> procurementContentList = procurementContentService.list(Wrappers.<ProcurementContent>query().eq("purchasing_plan_id", purchasingPlan.getPurchasingPlanId()));
            //如果采购计划的明细全部被删除，则自动删除这个采购计划
            if (procurementContentList.size() == 0) {
                this.removeById(purchasingPlan);
                return true;
            }
            calculatedAmount(procurementContent.getPurchasingPlanId());//计算经费预算
            return true;
        } else {
            return null;
        }
    }

    @Override//换货
    public ProcurementContent replacementById(PurchasingPlanDTO purchasingPlanDTO) {

        String replacementReagentConsumableId = purchasingPlanDTO.getReplacementReagentConsumableId();//换货试剂耗材ID
        ProcurementContent procurementContent = procurementContentService.getById(purchasingPlanDTO.getProcurementContentId());//采购计划明细
        //将换货信息补充至采购计划明细
        procurementContent.setReplacementReagentConsumableId(replacementReagentConsumableId);//换货试剂耗材ID
        procurementContent.setReplacementQuantity(purchasingPlanDTO.getReplacementQuantity());//换货数量
        procurementContent.setReplacementPrice(purchasingPlanDTO.getReplacementPrice());//换货单价
        procurementContent.setReplacementRemark(purchasingPlanDTO.getReplacementRemark());//换货备注
        procurementContent.setSupplierId(purchasingPlanDTO.getSupplierId());//换货供应商ID
        //关联这个采购计划明细的所有采购申请明细
        List<DetailsOfCentralized> list = detailsOfCentralizedService.list(Wrappers.<DetailsOfCentralized>query().eq("procurement_content_id", procurementContent.getProcurementContentId()));

        LambdaQueryWrapper<PurchaseListDetails> purchaseListDetailsLambdaQueryWrapper = new LambdaQueryWrapper<>();
        purchaseListDetailsLambdaQueryWrapper.eq(PurchaseListDetails::getProcurementContentId, procurementContent.getProcurementContentId());
        PurchaseListDetails purchaseListDetails = purchaseListDetailsService.getOne(purchaseListDetailsLambdaQueryWrapper);//查询是否有采购清单
        //如果存在采购清单，则需修改清单信息
        if (purchaseListDetails != null) {
            LambdaQueryWrapper<WarehousingContent> warehousingContentLambdaQueryWrapper = new LambdaQueryWrapper<>();
            warehousingContentLambdaQueryWrapper.eq(WarehousingContent::getPurchaseListDetailsId, purchaseListDetails.getId());

            purchaseListDetails.setSupplierId(purchasingPlanDTO.getSupplierId());//供应商ID
            purchaseListDetails.setPurchaseQuantity(purchasingPlanDTO.getReplacementQuantity());//采购数量
            purchaseListDetails.setReagentConsumableId(purchasingPlanDTO.getReplacementReagentConsumableId());//采购试剂耗材ID

            purchaseListDetailsService.updateById(purchaseListDetails);

            WarehousingContent warehousingContent = warehousingContentService.getOne(warehousingContentLambdaQueryWrapper);
            //若存在入库信息，则需修改入库记录
            if (warehousingContent != null) {

                if (warehousingContent.getWarehousingQuantity() == 0) {
                    warehousingContent.setReagentConsumableId(purchasingPlanDTO.getReplacementReagentConsumableId());//采购试剂耗材ID
                    warehousingContent.setTotalQuantity(purchasingPlanDTO.getReplacementQuantity());//采购数量
                    warehousingContent.setSupplierId(purchasingPlanDTO.getSupplierId());//供应商ID
                    ReagentConsumables reagentConsumables = reagentConsumablesService.getById(purchasingPlanDTO.getReplacementReagentConsumableId());
                    warehousingContent.setReagentConsumableName(reagentConsumables.getReagentConsumableName());//采购试剂耗材名称
                    warehousingContentService.updateById(warehousingContent);
                } else {
                    //如果已入库，则不能换货
                    throw new RuntimeException(String.format("该物品已被签收入库，无法进行换货处理"));
                }
            }
        }
        //将换货信息补充至每一个集中采购申请明细
        for (DetailsOfCentralized detailsOfCentralized : list) {
            detailsOfCentralized.setReplacementReagentConsumableId(replacementReagentConsumableId);
            detailsOfCentralized.setReplacementQuantity(purchasingPlanDTO.getReplacementQuantity());
            detailsOfCentralized.setReplacementRemark(purchasingPlanDTO.getReplacementRemark());
            detailsOfCentralizedService.updateById(detailsOfCentralized);
        }

        if (procurementContentService.updateById(procurementContent)) {
            return procurementContent;
        } else throw new RuntimeException(String.format("换货失败"));
    }

    @Override
    public PurchasingPlan commitById(String purchasingPlanId) {

        PurchasingPlan purchasingPlan = this.getById(purchasingPlanId);
        //若是审核审批不通过时提交，则需清空原有的审核审批信息
        //审核不通过
        if (purchasingPlan.getStatus() == -4) {
            purchasingPlan.setAuditOpinionOfPrimary("");
            purchasingPlan.setAuditResultOfPrimary(false);
            purchasingPlan.setPrimaryAuditorId("");
            purchasingPlan.setAuditTimeOfPrimary(null);
        }
        //审批不通过
        if (purchasingPlan.getStatus() == -5) {
            purchasingPlan.setAuditOpinionOfPrimary("");
            purchasingPlan.setAuditResultOfPrimary(false);
            purchasingPlan.setPrimaryAuditorId("");
            purchasingPlan.setAuditTimeOfPrimary(null);
            purchasingPlan.setApproverId("");
            purchasingPlan.setApprovalOpinion("");
            purchasingPlan.setApprovalResult(false);
            purchasingPlan.setApprovalTime(null);
        }

        purchasingPlan.setStatus(1);
        purchasingPlan.setCommitTime(LocalDateTime.now());//
        if (this.updateById(purchasingPlan)) {
            calculatedAmount(purchasingPlan.getPurchasingPlanId());
            return purchasingPlan;
        } else throw new RuntimeException(String.format("提交失败"));

    }


    @Override//审核采购计划
    public PurchasingPlan auditById(AuditAndApproveDTO auditAndApproveDto, DLPUser dlpUser) {

        PurchasingPlan purchasingPlan = this.getById(auditAndApproveDto.getUuId());

        if (purchasingPlan.getStatus() != 1) {
            throw new RuntimeException(String.format("当前状态不可审核"));
        }
        purchasingPlan.setAuditResultOfPrimary(auditAndApproveDto.getAuditResult());//审核结果
        purchasingPlan.setAuditOpinionOfPrimary(auditAndApproveDto.getAuditOpinion());//审核意见
        purchasingPlan.setAuditTimeOfPrimary(LocalDateTime.now());//审核时间
        purchasingPlan.setPrimaryAuditorId(dlpUser.getId());//审核人ID

        if (auditAndApproveDto.getAuditResult() == true) {
            purchasingPlan.setStatus(4);//审核通过
        } else {
            purchasingPlan.setStatus(-4);//审核不通过
        }
        if (this.updateById(purchasingPlan)) {
            return purchasingPlan;
        } else {
            return null;
        }
    }

    @Override//审批采购计划
    public PurchasingPlan approveById(AuditAndApproveDTO auditAndApproveDto, DLPUser dlpUser) {

        PurchasingPlan purchasingPlan = this.getById(auditAndApproveDto.getUuId());

        if (purchasingPlan.getStatus() != 4) {
            throw new RuntimeException(String.format("当前状态不可审批"));
        }
        purchasingPlan.setApprovalResult(auditAndApproveDto.getApproveResult());//审批结果
        purchasingPlan.setApprovalOpinion(auditAndApproveDto.getApproveOpinion());//审批意见
        purchasingPlan.setApprovalTime(LocalDateTime.now());//审批时间
        purchasingPlan.setApproverId(dlpUser.getId());//审批人ID
        //审批通过则创建采购清单
        if (auditAndApproveDto.getApproveResult() == true) {

            purchasingPlan.setStatus(5);//审批通过
            LambdaQueryWrapper<ProcurementContent> procurementContentLambdaQueryWrapper = new LambdaQueryWrapper<>();
            procurementContentLambdaQueryWrapper.eq(ProcurementContent::getPurchasingPlanId, purchasingPlan.getPurchasingPlanId());
            List<ProcurementContent> list = procurementContentService.list(procurementContentLambdaQueryWrapper);//采购计划明细

            PurchaseList purchaseList = new PurchaseList();//创建采购清单
            purchaseList.setId(IdWorker.get32UUID().toUpperCase());
            purchaseList.setName(DateTime.now() + "采购清单");//清单名称
            purchaseList.setStatus(0);//状态
            purchaseList.setType("采购计划");//清单类型，区分分散采购申请和采购计划
            purchaseList.setCreateBy(purchasingPlan.getCreateBy());//清单创建人
            purchaseListService.save(purchaseList);
            purchasingPlan.setPurchaseListId(purchaseList.getId());//关联采购清单

            for (ProcurementContent procurementContent : list) {

                PurchaseListDetails purchaseListDetails = new PurchaseListDetails();//采购清单明细
                purchaseListDetails.setPurchaseListId(purchaseList.getId());//关联采购清单
                purchaseListDetails.setPurchaseQuantity(procurementContent.getQuantityPurchased());//采购数量
                purchaseListDetails.setReagentConsumableId(procurementContent.getReagentConsumableId());//采购试剂耗材ID
                purchaseListDetails.setId(IdWorker.get32UUID().toUpperCase());
                purchaseListDetails.setProcurementContentId(procurementContent.getProcurementContentId());//关联采购计划明细
                purchaseListDetailsService.save(purchaseListDetails);
            }
        } else {
            purchasingPlan.setStatus(-5);//审核不通过
        }
        if (this.updateById(purchasingPlan)) {
            return purchasingPlan;
        } else {
            return null;
        }
    }

    //计算经费预算方法
    public PurchasingPlan calculatedAmount(String purchasingPlanId) {

        PurchasingPlan purchasingPlan = this.getById(purchasingPlanId);//采购计划

        LambdaQueryWrapper<ProcurementContent> procurementContentLambdaQueryWrapper = new LambdaQueryWrapper<>();
        procurementContentLambdaQueryWrapper.eq(ProcurementContent::getPurchasingPlanId, purchasingPlan.getPurchasingPlanId());
        List<ProcurementContent> list = procurementContentService.list(procurementContentLambdaQueryWrapper);//这个采购计划内的所有采购计划明细
        double x = 0;
        //计算每个采购计划明细的小计(单价*采购数量)
        for (
                ProcurementContent procurementContent : list) {
            x = x + procurementContent.getQuantityPurchased() * procurementContent.getUnitPrice();
        }
        purchasingPlan.setAppropriationBudget(x);//经费预算

        if (this.updateById(purchasingPlan)) {
            return purchasingPlan;
        } else throw new RuntimeException(String.format("计算失败"));
    }

    @Override//查看换货信息
    public ReplacementVO getReplacementVO(String id) {

        ProcurementContent procurementContent = procurementContentService.getById(id);//采购计划明细
        ReplacementVO replacementVO = new ReplacementVO();//换货VO

        //判断换货信息是否充分
        if (procurementContent.getReplacementQuantity() != null && procurementContent.getReplacementPrice() != null && procurementContent.getReplacementRemark() != null
                && procurementContent.getReplacementReagentConsumableId() != null && procurementContent.getSupplierId() != null) {

            SupplierInformation supplierInformation = supplierInformationService.getById(procurementContent.getSupplierId());//供应商
            ReagentConsumables reagentConsumables = reagentConsumablesService.getById(procurementContent.getReplacementReagentConsumableId());//试剂耗材信息

            replacementVO.setReplacementQuantity(procurementContent.getReplacementQuantity());//换货数量
            replacementVO.setReplacementRemark(procurementContent.getReplacementRemark());//换货备注
            replacementVO.setReplacementPrice(procurementContent.getReplacementPrice());//换货单价
            replacementVO.setReplacementReagentConsumableId(procurementContent.getReplacementReagentConsumableId());//换货试剂耗材ID
            replacementVO.setSupplierId(procurementContent.getSupplierId());//换货供应商ID
            replacementVO.setSupplierName(supplierInformation.getSupplierName());//换货供应商名称
            replacementVO.setReagentConsumables(reagentConsumables);//换货试剂耗材信息
        }
        return replacementVO;

    }
}
