package digital.laboratory.platform.reagent.service.impl;

import cn.hutool.core.date.LocalDateTimeUtil;
import cn.hutool.core.io.IoUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.deepoove.poi.XWPFTemplate;
import com.deepoove.poi.config.Configure;
import com.deepoove.poi.config.ConfigureBuilder;
import com.deepoove.poi.plugin.table.LoopRowTableRenderPolicy;
import digital.laboratory.platform.common.feign.RemoteWord2PDFService;
import digital.laboratory.platform.common.mybatis.security.service.DLPUser;
import digital.laboratory.platform.common.oss.service.OssFile;
import digital.laboratory.platform.reagent.dto.StandardSolutionCurveDTO;
import digital.laboratory.platform.reagent.entity.ReagentConsumables;
import digital.laboratory.platform.reagent.entity.ReferenceMaterial;
import digital.laboratory.platform.reagent.entity.StandardReserveSolution;
import digital.laboratory.platform.reagent.entity.StandardSolutionCurve;
import digital.laboratory.platform.reagent.mapper.StandardSolutionCurveMapper;
import digital.laboratory.platform.reagent.service.ReagentConsumablesService;
import digital.laboratory.platform.reagent.service.ReferenceMaterialService;
import digital.laboratory.platform.reagent.service.StandardReserveSolutionService;
import digital.laboratory.platform.reagent.service.StandardSolutionCurveService;
import digital.laboratory.platform.reagent.vo.StandardSolutionCurveVO;
import feign.Response;
import org.apache.commons.io.output.ByteArrayOutputStream;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.mock.web.MockMultipartFile;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.ByteArrayInputStream;
import java.time.LocalDateTime;
import java.util.Calendar;
import java.util.HashMap;
import java.util.List;

/**
 * (标准工作曲线溶液配置记录表)服务实现类
 *
 * @author Zhang Xiaolong created at  2023-03-10
 * @describe (标准工作曲线溶液配置记录表) 服务实现类
 */
@Service
@SuppressWarnings("all")
public class StandardSolutionCurveServiceImpl extends ServiceImpl<StandardSolutionCurveMapper, StandardSolutionCurve> implements StandardSolutionCurveService {

    @Autowired
    private StandardReserveSolutionService standardReserveSolutionService;

    @Autowired
    private OssFile ossFile;

    @Autowired
    private RemoteWord2PDFService remoteWord2PDFService;

    @Autowired
    private ReferenceMaterialService referenceMaterialService;

    @Autowired
    private ReagentConsumablesService reagentConsumablesService;

    @Override
    public IPage<StandardSolutionCurveVO> getStandardSolutionCurveVOPage(IPage<StandardSolutionCurve> page, QueryWrapper<StandardSolutionCurve> qw) {

        IPage<StandardSolutionCurveVO> standardSolutionCurveVOPage = baseMapper.getStandardSolutionCurveVOPage(page, qw);

        return standardSolutionCurveVOPage;
    }

    @Override
    public StandardSolutionCurve addById(StandardSolutionCurveDTO standardSolutionCurveDTO, DLPUser dlpUser) {

        StandardSolutionCurve standardSolutionCurve = new StandardSolutionCurve();

        StandardReserveSolution byId = standardReserveSolutionService.getById(standardSolutionCurveDTO.getSolutionId());

        if (byId.getStatus() != 1) {
            throw new RuntimeException(String.format("当前标准储备溶液还未入库，请先入库后再使用"));
        }

        ReferenceMaterial referenceMaterial = referenceMaterialService.getById(byId.getReferenceId());

        if (referenceMaterial.getStatus() != -3) {
            throw new RuntimeException(String.format("请先领用该标准储备溶液后，再进行操作"));

        } else if (!(referenceMaterial.getStatus() == 0 || referenceMaterial.getStatus() == -2||referenceMaterial.getStatus()==-3)) {

            throw new RuntimeException(String.format("该标准储备溶液已使用完毕或已停止使用/报废使用，不能进行操作"));

        }

        standardSolutionCurve.setId(IdWorker.get32UUID().toUpperCase());

        ReagentConsumables reagentConsumables = reagentConsumablesService.getById(referenceMaterial.getReagentConsumableId());

        standardSolutionCurve.setSolutionName(reagentConsumables.getReagentConsumableName());
        standardSolutionCurve.setSolutionConcentration(reagentConsumables.getConfigurationConcentration());
        standardSolutionCurve.setConfigurationDate(LocalDateTime.now());
        standardSolutionCurve.setConcentrationNumbering(referenceMaterial.getNumber());
        standardSolutionCurve.setStepSolutionConcentration(standardSolutionCurveDTO.getStepSolutionConcentration());
        standardSolutionCurve.setReferenceMaterialId(referenceMaterial.getId());
        standardSolutionCurve.setDispenserId(dlpUser.getId());

        if (this.save(standardSolutionCurve)) {
            return standardSolutionCurve;
        } else throw new RuntimeException(String.format("配置失败"));

    }

    @Override
    public void standardSolutionCurveTablePDF(List<StandardSolutionCurveVO> voList, String id, HttpServletRequest theHttpServletRequest, HttpServletResponse httpServletResponse) throws Exception {
        System.out.println("standardSolutionCurveTablePDF.................");

        Integer x = 1;
        for (StandardSolutionCurveVO standardSolutionCurveVO : voList) {

            standardSolutionCurveVO.setSerialnNumber(x.toString());

            standardSolutionCurveVO.setConfigurationDatePrint(LocalDateTimeUtil.format(standardSolutionCurveVO.getConfigurationDate(), "yyyy年MM月dd日"));

            x++;
        }

        String applyFileName = "标准工作曲线溶液配制记录表-" + id;
        //-----------------------------
        // 生成 word 版本的 验收记录表

        ByteArrayOutputStream bos = new ByteArrayOutputStream();

        ossFile.fileGet("template" + "/" + "标准工作曲线溶液配制记录表模板.docx", bos);

        byte[] templateArray = bos.toByteArray();

        ByteArrayInputStream bis = new ByteArrayInputStream(templateArray);

        bos.close();

        LoopRowTableRenderPolicy policy = new LoopRowTableRenderPolicy();

        ConfigureBuilder builder = Configure.builder().buildGramer("${", "}").useSpringEL(false);

        builder.bind("voList", policy).build();

        Calendar instance = Calendar.getInstance();
        int year = instance.get(Calendar.YEAR);
        /**
         * 循环表格
         */
        XWPFTemplate template = XWPFTemplate.compile(bis, builder.build()).render(
                new HashMap<String, Object>() {{
                    put("voList", voList);
                    put("year", year);
                    put("size", 1);


                }}
        );
        bis.close();

        ByteArrayOutputStream fosWord = new ByteArrayOutputStream();
        template.write(fosWord);
        template.close();

        //------------
        ByteArrayInputStream fisWord = new ByteArrayInputStream(fosWord.toByteArray());
        fosWord.close();

        //MockMultipartFile mockMultipartFile = new MockMultipartFile("file", entrustmentLetterFileName + ".docx", "application/vnd.openxmlformats-officedocument.wordprocessingml.document", fisWord);
        MockMultipartFile mockMultipartFile = new MockMultipartFile("file", applyFileName + ".docx", "image/jpg", fisWord);
        Response response = remoteWord2PDFService.word2pdf(mockMultipartFile);
        fisWord.close();


        ByteArrayOutputStream outPDF = new ByteArrayOutputStream();
        IoUtil.copy(response.body().asInputStream(), outPDF, IoUtil.DEFAULT_MIDDLE_BUFFER_SIZE);
        ByteArrayInputStream isPDF = new ByteArrayInputStream(outPDF.toByteArray());
        outPDF.close();


        ossFile.fileSave("document" + "/" + "standardSolutionCurve" + "/" + id + "/" + applyFileName + ".pdf", isPDF);
        isPDF.close();


        System.out.println(String.format("转换为 PDF 结束"));

    }

    @Override

    public StandardSolutionCurveVO getStandardSolutionCurveVO(String id) {

        StandardSolutionCurveVO standardSolutionCurveVO = baseMapper.getStandardSolutionCurveVO(id);

        return standardSolutionCurveVO;
    }

}
